package net.knarcraft.questsdynmap;

import me.blackvein.quests.QuestsAPI;
import org.bukkit.Bukkit;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.dynmap.DynmapAPI;
import org.dynmap.markers.MarkerAPI;
import org.dynmap.markers.MarkerIcon;
import org.dynmap.markers.MarkerSet;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

@SuppressWarnings("unused")
public final class DynmapCitizens extends JavaPlugin {

    private static DynmapCitizens instance;

    @Override
    public void onEnable() {
        instance = this;
        //Initialize quest and dynmap APIs
        PluginManager pluginManager = Bukkit.getPluginManager();
        Plugin dynmapPlugin = pluginManager.getPlugin("dynmap");
        if (!(dynmapPlugin instanceof DynmapAPI dynmapAPI) || dynmapAPI.getMarkerAPI() == null) {
            this.getLogger().log(Level.SEVERE, "Could not initialize Dynmap");
            this.onDisable();
            return;
        }

        QuestsAPI questsAPI = (QuestsAPI) Bukkit.getServer().getPluginManager().getPlugin("Quests");

        MarkerSet questMarkerSet = getMarkerSet(dynmapAPI, "quests", "Quests");
        MarkerSet questAreaMarkerSet = getMarkerSet(dynmapAPI, "quest_areas", "Quest areas");
        MarkerSet blacksmithSet = getMarkerSet(dynmapAPI, "blacksmiths", "Blacksmiths");
        if (questMarkerSet == null || questAreaMarkerSet == null || blacksmithSet == null) {
            return;
        }

        //TODO: Perhaps make this configurable
        questMarkerSet.setHideByDefault(false);
        questAreaMarkerSet.setHideByDefault(true);
        blacksmithSet.setHideByDefault(false);
        //TODO: Make every icon configurable
        MarkerAPI markerAPI = dynmapAPI.getMarkerAPI();
        Map<Icon, MarkerIcon> markerIcons = new HashMap<>();
        markerIcons.put(Icon.QUEST_GIVER, markerAPI.getMarkerIcon("exclamation"));
        markerIcons.put(Icon.QUEST_DELIVER, markerAPI.getMarkerIcon("basket"));
        markerIcons.put(Icon.QUEST_KILL, markerAPI.getMarkerIcon("skull"));
        markerIcons.put(Icon.QUEST_INTERACT, markerAPI.getMarkerIcon("comment"));
        markerIcons.put(Icon.BLACKSMITH, markerAPI.getMarkerIcon("hammer"));
        DynmapDrawer.initialize(questsAPI, questMarkerSet, questAreaMarkerSet, blacksmithSet, markerIcons);

        Bukkit.getScheduler().runTaskTimer(this, this::updateIcons, 10 * 20, 120 * 20);
    }

    @Override
    public void onDisable() {
        // Plugin shutdown logic
    }

    /**
     * Gets an instance of this plugin
     *
     * @return <p>An instance of this plugin</p>
     */
    public static DynmapCitizens getInstance() {
        return instance;
    }

    /**
     * Gets the given marker set (and creates it if necessary)
     *
     * @param dynmapAPI <p>A reference to dynmap's API</p>
     * @param setId     <p>The id of the marker set to get</p>
     * @param label     <p>The label of the marker set if creation is necessary</p>
     * @return <p>The marker set, or null if something went wrong</p>
     */
    private MarkerSet getMarkerSet(DynmapAPI dynmapAPI, String setId, String label) {
        MarkerSet markerSet = dynmapAPI.getMarkerAPI().getMarkerSet(setId);
        if (markerSet == null) {
            markerSet = dynmapAPI.getMarkerAPI().createMarkerSet(setId, label, null, false);
            if (markerSet == null) {
                this.getLogger().log(Level.SEVERE, "Unable to get or create " + setId + " marker set");
                this.onDisable();
                return null;
            }
        }
        return markerSet;
    }

    private void updateIcons() {
        DynmapDrawer.updateCitizensMarkers();
    }

}
